/* 
 * Interface.h
 *
 *  Created on: 17 NOV 2016
 *      Author: Roman Ryutin
 */

#ifndef INTERFACE_HH 
#define INTERFACE_HH

#include <fstream>
#include <cassert>
#include <cmath>
#include <complex>
#include <cstdlib>
#include <iostream>
#include <map>
#include <string>
#include <utility>
#include <vector>
#include <stdio.h>
#include <time.h>
#include <iomanip>
#include <limits>
#include <string>
#include <ctime>
 
#include "I.h"
#include "PI.h"
#include "newrand.h"
#include "NDarray.h"
#include "NDgrid.h"
#include "iFunc.h"
#include "Generator.h"
#include "Particle.h"
#include "Constants.h"
#include "Kinematics.h"
#include "Event.h"

namespace ExDiff{

// aux class for interface
class Interface{
  public:
    Interface():
 IDauthors(1),
 IDprocess(0),
 IDenergy(0),
 version(1),
 kinformat(0),
 Nevents(100),
 outformat(0) 
{
	assert(Nevents > 0);	
	assert(IDauthors > 0);
	assert(IDprocess >= 0);
	assert(IDenergy >= 0);
	assert(version > 0);
	assert(kinformat >= 0);
	assert(outformat >= 0);
}
//		
    Interface(const char* filename_):
 IDauthors(1),
 IDprocess(0),
 IDenergy(0),
 version(1),
 kinformat(0),
 Nevents(100),
 outformat(0) 
{
	assert(Nevents > 0);	
	assert(IDauthors > 0);
	assert(IDprocess >= 0);
	assert(IDenergy >= 0);
	assert(version > 0);
	assert(kinformat >= 0);
	assert(outformat >= 0);
	
	GetCard(filename_);
}
//	
	Interface(Interface& interface_){}
    virtual ~Interface(){}
	
// get IDauthors const
    inline const int& _IDauthors() const{
      return IDauthors;
    };	
// get IDauthors
    inline int& _IDauthors(){
      return IDauthors;
    };
	
// get IDprocess const
    inline const int& _IDprocess() const{
      return IDprocess;
    };	
// get IDprocess
    inline int& _IDprocess(){
      return IDprocess;
    };	

// get IDenergy const
    inline const int& _IDenergy() const{
      return IDenergy;
    };	
// get IDenergy
    inline int& _IDenergy(){
      return IDenergy;
    };

// get version const
    inline const int& _version() const{
      return version;
    };	
// get version
    inline int& _version(){
      return version;
    };
	
// get Nevents const
    inline const int& _Nevents() const{
      return Nevents;
    };	
// get Nevents
    inline int& _Nevents(){
      return Nevents;
    };	
	
// get format const
    inline const int& _kinformat() const{
      return kinformat;
    };	
// get format
    inline int& _kinformat(){
      return kinformat;
    };		
	
// get format const
    inline const int& _outformat() const{
      return outformat;
    };	
// get format
    inline int& _outformat(){
      return outformat;
    };			
	
	
// get energy by IDenergy
    inline double Get_Energy(int _IDenergy){
      double res;
	  if(_IDenergy == 0){res = 13000.0;}
	  if(_IDenergy == 1){res = 2760.0;}
	  if(_IDenergy == 2){res = 7000.0;}
	  if(_IDenergy == 3){res = 8000.0;}
	  if(_IDenergy == 4){res = 14000.0;}
	  return res;
    };	
	
// get authors by IDauthors
    inline std::string Get_Authors(int _IDauthors){
      std::string res;
	  if(_IDauthors == 1){res = "A.A. Godizov";}
	  if(_IDauthors == 2){res = "R.A. Ryutin";}
	  if(_IDauthors == 3){res = "V.A. Petrov, R.A. Ryutin";}
	  return res;
    };	

// get process by IDprocess
    inline std::string Get_Process(int _IDprocess){
      std::string res;
	  if(_IDprocess == 0){res = "p + p -> p + p (elastic scattering)";}
	  if(_IDprocess == 1){res = "p + p -> p + f_0(1710) + p (exclusive meson central production)";}
	  if(_IDprocess == 2){res = "p + p -> p + f_2(1950) (total) + p (exclusive meson central production)";}
          if(_IDprocess == 20){res = "p + p -> p + f_2(1950) (S=0) + p (exclusive meson central production)";}
          if(_IDprocess == 21){res = "p + p -> p + f_2(1950) (S=1) + p (exclusive meson central production)";}
          if(_IDprocess == 22){res = "p + p -> p + f_2(1950) (S=2) + p (exclusive meson central production)";}
	  if(_IDprocess == 3){res = "p + p -> p + pi + pi + p (exclusive di-pion central production)";}	
          if(_IDprocess == 4){res = "p + p -> p + f_0(1500) + p (exclusive meson central production)";} 	  
	  return res;
    };		

// print information about version of the generator
    inline void GeneratorInfo(){
       std::cout << "+++++++++++++++++++++++++++++++++++++++++++++++" << std::endl;
       std::cout << "########            #####" << std::endl;  
       std::cout << "#                   #    #" << std::endl;
       std::cout << "#                   #     #        ###    ###" << std::endl;
       std::cout << "#                   #      #      #   #  #   #" << std::endl;
       std::cout << "########  #     #   #      #  #   #      #" << std::endl; 
       std::cout << "#          #   #    #      #     ###    ###" << std::endl;
       std::cout << "#           ###     #     #   #   #      #" << std::endl; 
       std::cout << "#          #   #    #    #    #   #      #" << std::endl;
       std::cout << "########  #     #   #####     #   #      #" << std::endl;
       std::cout << "+++++++++++++++++++++++++++++++++++++++++++++++" << std::endl;
       std::cout << "     Exclusive         Diffractive PROcesses" << std::endl;
       std::cout << "===============================================" << std::endl;
       std::cout << "================ VERSION 1.0 ==================" << std::endl;
       std::cout << "===============================================" << std::endl;
       std::cout << "==== by Roman Ryutin and Anton Godizov ========" << std::endl;
       std::cout << "==== December 1 2016 ==========================" << std::endl;
       std::cout << "===============================================" << std::endl;
       std::cout << "\n\n" << std::endl;	    
       return;	 
	}	
// print information about process and kinematics, authors, version etc.	
	void PrintPars();
// reading info from card file	
	void GetCard(const char*);
// for generation and reformatting
	void GenerateFile();
//	void GenerateFileStandard(int); // or GenerateFile reload !
//	void ReFormatFile(int, const char*, int, const char*);
	
	inline std::string ResultFileName(){
		std::string str3;

        std::stringstream out3;
        out3 << "output/A" << IDauthors << "M" << IDprocess << "E" << IDenergy << "_" << version << "F" << kinformat << ".exdiff";
        str3 = out3.str();
//		std::cout << str <<std::endl;
		return str3;	
	}		

// typical grids
// general grid {x1,x2,...} : {x1min,x2min,...} -> {x1max,x2max,...} step {1/(v[1]-1),1/(v[2]-1),...}; v ={N1,N2, ...};
    inline NDgrid<double> grid_1(const std::vector<size_t>& dim_, const std::vector<double>& xmin_, const std::vector<double>& xmax_){
		
		NDgrid<double> res(dim_);
		
		std::size_t i = 0;
		std::size_t j = 0;
		
//		std::cout << "begin calc grid_1===========================================" << std::endl;
		
        for(i = 0; i < dim_.size(); i++){		
  		  for(j = 0; j < dim_[i]; j++){
		    res.GetVar(i,j) = xmin_[i]+j*(xmax_[i]-xmin_[i])/(dim_[i]-1);	
//            std::cout << res.GetVar(i,j) <<std::endl;			
		  }
		}  

        return res;		  
	} 	
// {t1,t2,phi12,log(xi1)}; t1, t2: 0 -> 1 step 1/128.0, phi12: 0 -> PI step PI/8; log(xi1): lmin -> lmax step (lmax-lmin)/8.0	
    inline NDgrid<double> grid_2(const double& mc_, const double& ximin_, const double& sqs_){	

//		std::cout << "begin calc grid_2===========================" << std::endl;
		
		double lmin = log(ximin_);
		double lmax = log(mc_*mc_/ximin_/sqs_/sqs_);
		
		std::vector<double> xmin_ = {0.0,0.0,0.0,lmin};
		std::vector<double> xmax_ = {1.0,1.0,PI,lmax};
		std::vector<size_t> dim_ = {129,129,9,9};
		
	    NDgrid<double> res(dim_); 
		res = grid_1(dim_,xmin_,xmax_);
		
        return res;		
	} 

	inline std::string DatFileName(){
		std::string str1;
		
        std::stringstream out1;
        out1 << "modeldata/A" << IDauthors << "M" << IDprocess << "E" << IDenergy << "_" << version << "F" << kinformat << ".dat";
        str1 = out1.str();
		
		return str1;			
	}
	inline std::string GridFileName(){ 
		std::string str2; 

        std::stringstream out2;
        out2 << "modeldata/A" << IDauthors << "M" << IDprocess << "E" << IDenergy << "_" << version << "F" << kinformat << ".grid";
        str2 = out2.str();
		
		return str2;	
	}		
	
 	   
  private:
    int IDauthors, IDprocess, IDenergy, version, Nevents, kinformat, outformat; 
// format = set of variables for kinematics
	


}; // end of class Interface definitions

}// end of namespace ExDiff

#endif /* INTERFACE_HH */
