/*
 * Interface.cpp
 *
 *  Created on: 17 NOV 2016
 *      Author: Roman Ryutin
 */

#include <fstream>
#include <cassert>
#include <cmath>
#include <complex>
#include <cstdlib>
#include <iostream>
#include <map>
#include <string>
#include <utility>
#include <vector>
#include <stdio.h>
#include <time.h>
#include <iomanip>
#include <limits>
#include <string>
#include <ctime>

#include "../inc/I.h"
#include "../inc/PI.h"
#include "../inc/newrand.h"
#include "../inc/NDarray.h"
#include "../inc/NDgrid.h"
#include "../inc/iFunc.h"
#include "../inc/Generator.h"
#include "../inc/Particle.h"
#include "../inc/Constants.h"
#include "../inc/Kinematics.h"
#include "../inc/Event.h"
#include "../inc/Interface.h"

// functions of class Interface ===========================================
///////////////////////////////////////////////////////////////////////////
void ExDiff::Interface::PrintPars(){ 
	std::cout << "========= Parameters for the generation ===========" <<std::endl;	
	std::cout << "Authors: " << Get_Authors(IDauthors) <<std::endl;
	std::cout << "Process: " << Get_Process(IDprocess) <<std::endl;	
	std::cout << "C.M. energy: " << Get_Energy(IDenergy) << " GeV" <<std::endl;	
	std::cout << "version: " << version << " format: " << kinformat <<std::endl;
	std::cout << "Number of events: " << Nevents << " Output format: " << outformat <<std::endl; 
	std::cout << "===================================================" <<std::endl;		
	return; 
};
/////////////////////////////////////////////////////////////////////////// 
void ExDiff::Interface::GetCard(const char *_filename){
	std::ifstream f; 
	 
	std::vector<int> pars;
	pars.clear();

    f.open(_filename, std::ifstream::in);
    if (!f.is_open()) {
       std::cout << "Can not open a *.card file!\n";   
	}	
	
	int par = 0;
	for(int i = 0; i < 7; i++){
	  f >> par;
      pars.push_back(par);	  
	}
// IDauthors IDprocess IDenergy version Nevents kinformat outformat	
    IDauthors = pars[0];
	IDprocess = pars[1];
	IDenergy = pars[2];
	version = pars[3];
	Nevents = pars[4];
	kinformat = pars[5];
	outformat = pars[6];	
    
	f.close();
	
	pars.clear();

    return;	
};
///////////////////////////////////////////////////////////////////////////

// generate file of events
void ExDiff::Interface::GenerateFile(){
	int flag = 0; // grid from file, if !=0 - special version of the grid
    int i = 0;
	ExDiff::Constants c;
	std::vector<std::size_t> dim;
// create NDarray, NDgrid using parameters IDauthors, IDprocess, version 
   if(IDauthors == 1){

      if(IDprocess == 1  || IDprocess == 2 || IDprocess == 4 || IDprocess > 19){
        if(version == 1 || version == 2){
           dim = {129,129,9,9};
		   flag = 1; // uniform grid version 1 by cycle
        }  		
// example of other versions, 1 is default version	  
//      if(version == 2){
//         std::vector<std::size_t> dim({400,400,20,20});
//      }    
	   
	  }   
	  if(IDprocess == 0){
         if(version == 1){
           dim = {501};
         } 			
	  }
	   	   
   }
   
   ExDiff::NDarray<double> dat_(dim);
   ExDiff::NDgrid<double>  grid_(dim);
	   
// create names of dat and grid files
   std::string datfile = DatFileName();    

// read NDarray, NDgrid from files, or, NDgrid create inside this function
//   std::cout << "dat file = " << datfile << std::endl;
   dat_.LoadFromFile(datfile);
   
   double mc_ = 1.0; 
   int id_ = 0;  
   double sqs_ = Get_Energy(IDenergy); 

   std::string gridfile = GridFileName();  
//   std::cout << "grid file = " << gridfile << std::endl;   
   
   if(flag != 0){
//	  printf("Fill Grid, flag = %1i - 1", flag); 
// fill the uniform grid with numbers
     if(flag == 1){
      double ximin_ = 0.0001;

      if(IDprocess == 1){ 
	    mc_ = 1.71; 
		id_ = c.ID1710;
        grid_ = grid_2(mc_, ximin_, sqs_);   
	  }  	 
      if(IDprocess == 2 || IDprocess == 20 || IDprocess == 21 || IDprocess == 22){ 
       mc_ = 1.95; 
	   id_ = c.ID1950;
//	   printf("Fill Grid, flag = %1i - 2\n", flag); 
       grid_ = grid_2(mc_, ximin_, sqs_ ); 
//   	   printf("Fill Grid, flag = %1i - 3\n", flag); 	   
	  }	 
      if(IDprocess == 4){ 
	    mc_ = 1.5; 
		id_ = c.ID1500;
        grid_ = grid_2(mc_, ximin_, sqs_);   
	  }	  
     }	  
	   
   } else{    
//      printf("Load Grid From File, flag = %1i", flag);
      grid_.LoadFromFile(gridfile);
   } 

//    printf("Fill Grid, flag = %1i - 4\n", flag); 
//	grid_.printC(); 
// create iFunc
    ExDiff::iFunc fungen(dat_,grid_);
// create Generator	
    ExDiff::Generator gen(fungen); 
    ExDiff::newrand();
	
	printf("TOTAL C-S. = %.6f \n", gen.GetFUNGTOT());
	
// generate 1 sample
    std::vector<double> genvec;	
	genvec = gen.Generate();
// additional phi variable;	
//    fungen.PrintVector(genvec,genvec.size()); 

	genvec.push_back(2.0*PI*gen.RNUM());
	
//	fungen.PrintVector(genvec,genvec.size()); 
// for 2 -> 4 additional Eta_j(dat) and Phi_j(uniform) variables	
	
// create KinCM object
// here we use default set of variables
// 2 -> 2 {t}; phi
// 2 -> 3 {DeltaT_1, DeltaT_2, phi_12, log(xi_1)}; phi_1
    std::vector<double> masses__;
	std::vector<int> ids__;
	
    if(IDprocess == 1 || IDprocess == 2 || IDprocess == 4 || IDprocess > 19){  	
	  masses__ = {c.m_p,mc_,c.m_p};
	  ids__ = {c.IDproton,id_,c.IDproton};	  	
	}
    if(IDprocess == 0){
	  masses__ = {c.m_p,c.m_p};
	  ids__ = {c.IDproton,c.IDproton};	
	}	

	ExDiff::KinCM kinematics(sqs_,0,genvec,masses__,ids__);
// create Event object
    ExDiff::Event ev(kinematics.EvParticles());
//	ev.Print(2);

    std::string outputfile = ResultFileName();
	
//    std::cout << "output file = " << outputfile << std::endl;	
//	ev.AddToFile(outformat,outputfile);	

//    kinematics.Print();

//    PrintPars();   

// time variables ////////////////////////////////////////
    std::clock_t start;
    double duration;
/////////////////////////////////////////////////////////


	
	for(i = 0; i < Nevents; i++){

// time for calculations start	//////////////////////////
			if(i == 0){													
			  printf("clock start = %d\n", std::clock());
			  start = std::clock();
			}
/////////////////////////////////////////////////////////		
		
		
		genvec.clear();
		
		genvec = gen.Generate(); 
		
	    genvec.push_back(2.0*PI*gen.RNUM());

		kinematics.ResetVars(genvec); 	
		 
//		kinematics.Print();
//		std::cout << "Phys() = " << kinematics.Phys() << "   flag = " << kinematics._flag() << std::endl;

		ev.Reset(kinematics.EvParticles());
		
//		ev.Print(2);
		
//		std::cout << "outformat = " << outformat << std::endl;
//		std::cout << "output file = " << outputfile << std::endl;

// FOR DEVELOPERS ================================================================================
// You can use directly all the parameters of particles in the event
// to make your own output or connection to your software
// instead of the string 
		
		ev.AddToFile(outformat,outputfile);		
		
// use your own method with the following parameters of all particles in the event
// i: 0 => ev._particles().size()-1 [i changes 0=>3 (2 to 2), 0=>4 (2 to 3) and 0=>5 (2 to 4)]
// ev._particles().size() = number of particles in the event
// ev._particles()[i]._ID() = ID of the	particle
// ev._particles()[i]._m() = mass of the	particle
// ev._particles()[i]._E() = energy of the	particle
// ev._particles()[i]._px() = px of the	particle
// ev._particles()[i]._py() = py of the	particle
// ev._particles()[i]._pz() = pz of the	particle	
// FOR DEVELOPERS ================================================================================
		
// time for calculations end ///////////////////////////////////////////////			 				
            if(i == Nevents - 1){ 
			printf("clock stop = %d\n", std::clock());
            duration = ( std::clock() - start ) / (double) CLOCKS_PER_SEC;
            std::cout<<"duration = "<< duration << '\n';				
			}		
////////////////////////////////////////////////////////////////////////////		

		
	}	

    return;	
	
};

